"""collapsible_panel"""
from dash import html


def collapsible_panel(title, children, default_open=False):
    """Returns a Dash HTML component that allows the user to open and close a
    collapsible panel containing children"""
    return html.Details(
        [
            html.Summary(
                html.Span(
                    title,
                    className="govuk-!-margin-0 govuk-details__summary-text",
                ),
                className="govuk-details__summary govuk-!-margin-0",
                **{"data-module": "govuk-details"}
            ),
            html.Div(
                children,
                className="govuk-!-padding-0 govuk-!-margin-top-1",
                style={
                    "display": "flex",
                    "alignContent": "stretch",
                    "alignItems": "flex-end",
                    "flexFlow": "row wrap",
                    "justifyContent": "space-between",
                    "gap": "1rem",
                    "margin": "0",
                    "padding": "0",
                },
            ),
        ],
        className="govuk-details govuk-!-margin-0",
        **{"open": default_open}
    )



"""header"""

from dash import html


def header(title):
    """
    The header component, shared across all dashboard views.

    Based on the header component provided by the GOV.UK Design System.
    https://design-system.service.gov.uk/components/header/
    """
    return html.Header(
        html.Div(
            html.Div(
                [
                    html.Img(
                        src="assets\\images\\DLUHC_WHITE_Master_AW_sm.png",
                        srcSet="assets\\images\\DLUHC_WHITE_Master_AW_sm.png 490w",
                        sizes="(min-width: 600px) 200px, 30vw",
                        className="header-image",
                        style={"maxWidth": "200px"},
                        alt="Department for Levelling Up, Housing & Communities",
                    ),
                    html.A(
                        title,
                        href="/",
                        className=" ".join(
                            [
                                "govuk-header__link",
                                "govuk-header__link--service-name",
                                "dashboard-title",
                            ],
                        ),
                    ),
                    html.Strong(
                        "OFFICIAL",
                        className="govuk-tag protective-marking",
                        id="protective-marking",
                    ),
                    html.Button(
                        "Menu ▼",
                        id="mobile-menu-btn",
                        className="mobile-menu-button govuk-button",
                    ),
                ],
                className="govuk-header__content",
            ),
            className="govuk-header__container govuk-width-container",
        ),
        className="govuk-header",
        role="banner",
        **{"data-module": "govuk-header"},
    )



"""Line chart function"""
from typing import Optional
import plotly.express as px
from gov_uk_dashboards.axes import calc_axis_range
from gov_uk_dashboards.colours import ONSAccessibleColours
from .styles import LineStyle
from .chart_data import ChartData


def line_chart(
    data: ChartData,
    title: str,
    markers: bool = False,
    line_styles: Optional[dict[str, LineStyle]] = None,
    **px_line_kwargs,
):
    """
    Create and return a plotly express line chart with standard formatting.

    Dataframe should be sorted so x axis is in the correct order for plotting.

    If no style information provided, lines will be plotted as solid lines
    using the ONSAcessibleColours enum for their colours.

    Args:
        data (ChartData): Data for the chart.
        title (str): Title to be shown above the chart.
        markers (bool, optional): Whether markers should be plotted for each point.
            Defaults to False.
        line_styles (dict[str, LineStyle], optional): A dictionary with keys that match
            the categories in the category column (if supplied), and values that are
            LineStyle data objects to set out the style of the corresponding line.
            Defaults to None.
        **px_line_kwargs: Any other keyword arguments to pass to the plotly express
            line graph function.

    Returns:
        plotly.Figure: The generated line chart figure object.
    """
    color_discrete_map = None
    line_dash_map = None
    labels = None
    if line_styles:
        color_discrete_map = {
            category: line_style.color for category, line_style in line_styles.items()
        }
        line_dash_map = {
            category: line_style.dash_pattern
            for category, line_style in line_styles.items()
        }
        # If line dashes are set, plotly express automatically appends the name of the dash style
        # to the label in the legend/hover data.
        # As this is normally not desired, labels are set manually to override this.
        labels = {category: category for category in line_styles}

    linechart = px.line(
        data.dataframe,
        x=data.x_column,
        y=data.y_column,
        range_y=calc_axis_range(data.dataframe, data.y_column),
        color_discrete_map=color_discrete_map,
        line_dash_map=line_dash_map,
        line_dash=data.category_column,
        color=data.category_column,
        labels=labels,
        markers=markers,
        color_discrete_sequence=[colour.value for colour in ONSAccessibleColours]
        if not line_styles
        else None,
        **px_line_kwargs,
    )

    linechart.update_layout(
        title=title,
        paper_bgcolor="rgba(0,0,0,0)",
        plot_bgcolor="rgba(0,0,0,0)",
        xaxis_type="category",
    )

    return linechart



"""details"""
from dash import html


def details(details_summary: str, details_text: str) -> html.Details:
    """
    HTML component for showing a summary expandable with more information beneath.

    Makes a page easier to scan by letting users reveal more detailed information
    only if they need it.

    Part of the Gov.UK Design System:
    https://design-system.service.gov.uk/components/details/

    Args:
        details_summary (str): The summary text to display all the time.
        details_text (str): The details to reveal when the use requests them.

    Returns:
        html.Details: A Details HTML element with the specified text
    """
    return html.Details(
        [
            html.Summary(
                [
                    html.Span(
                        details_summary,
                        className="govuk-details__summary-text",
                    )
                ],
                className="govuk-details__summary",
            ),
            html.Div(details_text, className="govuk-details__text"),
        ],
        className="govuk-details",
    )



"""card"""
from dash import html


def card(children):
    """A rectangle with a grey background.
    Mostly used to wrap individual visualisations, e.g. a gauge."""
    return html.Div(children, className="mini-card")


def empty_card():
    """
    An empty card that is hidden to help keep alignment with rows and columns.
    """
    return html.Div(className="mini-card-empty")



"""
Read government dashboard template
"""
import os


def read_template(favicon_path: str = r"assets\images\gov_favicon.ico") -> str:
    """
    Read the government html template.
    Args:
        favicon_path (str): Optional file path to favicon.
    :return: String version of the template.
    """
    path = os.path.dirname(__file__)
    path = os.path.join(path, "template.html")
    with open(path, encoding="utf-8") as file:
        template = file.read()
        return template.replace("{{favicon_path}}", favicon_path)



"""ChartData dataclass"""
from dataclasses import dataclass
from typing import Optional

import pandas as pd


@dataclass
class ChartData:
    """
    Dataclass containing standard information useful for plotting charts.

    Attributes:
        dataframe (pd.DataFrame): The dataframe containing the data for the chart.
        x_column (str): The label of the column containing the x axis values.
        y_column (str): The label of the column containing the y axis values.
        category_column (str, optional): If there are multiple categories in the data,
            this column gives the value to categorize them. Defaults to None.
    """

    dataframe: pd.DataFrame
    x_column: str
    y_column: str
    category_column: Optional[str] = None



"""download_button"""
import warnings
from dash import html


def download_button(button_text: str, button_id: str = "download-button"):
    """
    Return a download button which is aligned to the right

    Args:
    button_text (str): The text to display on the button.
    button_id: (str, Optional) = id for dropdown, default to "download-button"
    """
    warnings.warn(
        "Note there is an alternative function to download_button() called "
        "download_button_with_icon() which includes a download icon and improved styling.",
        Warning,
        stacklevel=2,
    )

    return html.Div(
        [
            html.Button(
                button_text,
                id=button_id,
                n_clicks=0,
                className="govuk-button govuk-button--secondary",
            ),
        ],
        className="govuk-button-group",
        style={"float": "right"},
    )


def download_button_with_icon(button_text: str, button_id: str = "download-button"):
    """
    Return a download button styled with additional classes and an SVG icon,
    aligned to the right.

    Args:
    button_text (str): The text to display on the button.
    button_id (str, Optional): ID for the button, defaults to "download-button".
    """
    classes = (
        "govuk-button govuk-button--primary "
        "govuk-!-margin-bottom-0 govuk-!-margin-top-4 "
        "flex w-auto items-center gap-2 print:hidden"
    )
    return html.Div(
        [
            html.Button(
                [html.Div("", className="download-icon"), button_text],
                id=button_id,
                n_clicks=0,
                className=classes,
                type="submit",
                style={"display": "flex", "alignItems": "center", "gap": "8px"},
            ),
        ],
        className="govuk-button-group",
        style={"float": "right"},
    )



"""Context_banner"""
from dash import html


def context_banner():
    """
    Return context banner based on Gov UK Design component notification banner component.
    """
    banner = html.Div(
        [
            html.Div(
                [
                    html.P(
                        [
                            "Data on its own does not present a complete picture. This explorer"
                            " should be used to generate questions and not reach judgements."
                            " See about the data section for more information.",
                        ],
                        className="govuk-warning-text__text notification-banner-under-filter-panel",
                    )
                ],
                className="govuk-notification-banner__content",
            ),
        ],
        className="govuk-notification-banner",
        role="region",
        style={"maxWidth": "1000px", "margin-top": "20px", "margin-bottom": "20px"},
    )
    return banner



"""Module containing functions related to graph axes.

Contains:
- calc_axis_range: Return a range for a column of a dataframe so the derived
    axis is at the origin or the lowest negative value in the column, whichever
    is lower.
"""
from math import floor, ceil
import pandas as pd


def calc_axis_range(dataframe: pd.DataFrame, column: str) -> list:
    """Show origin on dashboard axis or negative value for numeric type column"""
    range_multiplier = 1.01
    axis_range = [
        floor(dataframe[[column]].min().iloc[0] * range_multiplier),
        ceil(dataframe[[column]].max().iloc[0] * range_multiplier),
    ]
    if axis_range[0] < 0:
        return axis_range
    return [0, axis_range[1]]



"""row_component"""
from dash import html


def row_component(cards, horizontal_scroll=None):
    """
    Creates a horizontal row used to contain cards.
    The card and row_component work together to create a
    layout that stretches and shrinks when the user changes the size of the window,
    or accesses the dashboard from a mobile device.

    See https://developer.mozilla.org/en-US/docs/Learn/CSS/CSS_layout for more information.
    """
    style = {"alignItems": "stretch"}
    if horizontal_scroll:
        style["overflow-x"] = "auto"

    return html.Div(cards, className="govuk-list card-container", style=style)
